"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

class TripCreatedV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VEHICLE_ID_FIELD_NUMBER: builtins.int
    USER_ID_FIELD_NUMBER: builtins.int
    VIN_FIELD_NUMBER: builtins.int
    vehicle_id: typing.Text = ...
    user_id: typing.Text = ...
    vin: typing.Text = ...
    def __init__(self,
        *,
        vehicle_id : typing.Optional[typing.Text] = ...,
        user_id : typing.Optional[typing.Text] = ...,
        vin : typing.Optional[typing.Text] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["user_id",b"user_id","vehicle_id",b"vehicle_id","vin",b"vin"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["user_id",b"user_id","vehicle_id",b"vehicle_id","vin",b"vin"]) -> None: ...
global___TripCreatedV1 = TripCreatedV1

class TripCreatedV2(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VEHICLE_ID_FIELD_NUMBER: builtins.int
    VIN_FIELD_NUMBER: builtins.int
    vehicle_id: typing.Text = ...
    vin: typing.Text = ...
    def __init__(self,
        *,
        vehicle_id : typing.Optional[typing.Text] = ...,
        vin : typing.Optional[typing.Text] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["vehicle_id",b"vehicle_id","vin",b"vin"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["vehicle_id",b"vehicle_id","vin",b"vin"]) -> None: ...
global___TripCreatedV2 = TripCreatedV2

class DeviceCreatedV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    DEVICE_ID_FIELD_NUMBER: builtins.int
    USER_ID_FIELD_NUMBER: builtins.int
    device_id: typing.Text = ...
    user_id: typing.Text = ...
    def __init__(self,
        *,
        device_id : typing.Optional[typing.Text] = ...,
        user_id : typing.Optional[typing.Text] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["device_id",b"device_id","user_id",b"user_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["device_id",b"device_id","user_id",b"user_id"]) -> None: ...
global___DeviceCreatedV1 = DeviceCreatedV1

class VehicleCreatedV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    VEHICLE_ID_FIELD_NUMBER: builtins.int
    vehicle_id: typing.Text = ...
    def __init__(self,
        *,
        vehicle_id : typing.Optional[typing.Text] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["vehicle_id",b"vehicle_id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["vehicle_id",b"vehicle_id"]) -> None: ...
global___VehicleCreatedV1 = VehicleCreatedV1
